const COMPATIBLE_APP_ID = [1193,77,1105,1194,1222,1491,1750];

async function GetKeyData(kscApi, serial) {
    const response = await kscApi.LicenseKeys.GetKeyData({
        pKeyInfo: {'KLLIC_SERIAL': serial, 'KLLICSRV_KEYDATA': true, 'KLLIC_APP_IDS': true}
    })
    return response.result;
}

async function IsSetKpsnMode(kscApi) {
    const eulas = await kscApi.KsnInternal.GetNKsnEulas();
    return (eulas.result!==undefined && eulas.result.pNKsnEulas!==undefined && eulas.result.pNKsnEulas.length > 0 ) ? true : false;
}

async function IsSetHostedMode(kscApi) {
    if(kscApi.utils!==undefined && kscApi.utils.getWebConsoleInfo!==undefined) {
        const info = await kscApi.utils.getWebConsoleInfo();
        return info.isHosted!==undefined ? info.isHosted : false;
    }
    return false;
}

async function EnumCompatibleLicenseKeys(kscApi) {
    const licFields = ['KLLIC_APP_ID', 'KLLIC_LIMIT_DATE','KLLIC_KEY_TYPE','KLLIC_SERIAL','KLLICSRV_KEY_INSTALLED'];
    const response = await kscApi.LicenseKeys.EnumKeys({ pFields: licFields });
    const enumlics = response.result.filter(
        lic => lic.KLLICSRV_KEY_INSTALLED===true && new Date(lic.KLLIC_LIMIT_DATE.value) > new Date(),
    );

    const productKeys = await Promise.all(
        enumlics.map(( license ) => GetKeyData(kscApi, license.KLLIC_SERIAL))
    );
    const compatibleLics = productKeys.filter(lic => {
        for(let appId of Object.keys(lic.KLLIC_APP_IDS)){
            if(COMPATIBLE_APP_ID.includes(parseInt(appId)))
                return true;
        }
        return false;
    });

    return compatibleLics;
}

module.exports = {
    IsSetKpsnMode,
    IsSetHostedMode,
    EnumCompatibleLicenseKeys
}