const buffer = require('buffer');
const sections = ['ConnectionPolicy','BLPolicy'];

function getPswdrData(pswrd) {
    let bytes = Buffer.from(pswrd + String.fromCharCode(0), 'utf-8');
    let bufStr16 = buffer.transcode(bytes, 'utf8', 'utf16le');
    return bufStr16.toString('base64');
}

function toView (policiesCache) {
    const connOptions = policiesCache.ConnectionPolicy.Policy.KLPRSS_Val.ConnectionOptions.KLPRSS_Val[0];
    const DefaultProxy = connOptions.DefaultProxy.KLPRSS_Val;
    const ProxyAuth = DefaultProxy.ProxyAuth.KLPRSS_Val;

    let data = {
        ConnectionOptions: policiesCache.ConnectionPolicy.Policy.KLPRSS_Val.ConnectionOptions,
        ProxyMode: connOptions.ProxyUsage.KLPRSS_Val,
        ProxyServer: DefaultProxy.ProxyServer.KLPRSS_Val,
        ProxyPort: DefaultProxy.ProxyPort.KLPRSS_Val,
        BypassLocalAddresses: connOptions.BypassLocalAddresses.KLPRSS_Val,
        ProxyAuthMode: ProxyAuth.AuthType.KLPRSS_Val,
        ProxyAuthUser: ProxyAuth.AuthUser.KLPRSS_Val,
        ProxyPassword: ProxyAuth.AuthPassword.KLPRSS_Val,
        NewPassword: '',
        UseEmptyPassword: ProxyAuth.AuthPassword.KLPRSS_Val!=='' ? false : true,
        PasswordWasSet: ProxyAuth.AuthPassword.KLPRSS_Val!=='' ? true : false,
        UseNagentAsActivationProxy: policiesCache.BLPolicy.Policy.KLPRSS_Val.UseNagentAsActivationProxy
    }

    return data;
}

async function toRawEx (kscApi, changedData, policiesCache) {
    let connOptions = changedData.ConnectionOptions.KLPRSS_Val[0];
    let DefaultProxy = connOptions.DefaultProxy.KLPRSS_Val;
    connOptions.ProxyUsage.KLPRSS_Val = changedData.ProxyMode;
    connOptions.BypassLocalAddresses.KLPRSS_Val = changedData.BypassLocalAddresses;
    DefaultProxy.ProxyServer.KLPRSS_Val = changedData.ProxyServer;
    DefaultProxy.ProxyPort.KLPRSS_Val = changedData.ProxyPort;
    DefaultProxy.ProxyAuth.KLPRSS_Val.AuthType.KLPRSS_Val = changedData.ProxyAuthMode;
    DefaultProxy.ProxyAuth.KLPRSS_Val.AuthUser.KLPRSS_Val = changedData.ProxyAuthUser;
    if(changedData.UseEmptyPassword){
        DefaultProxy.ProxyAuth.KLPRSS_Val.AuthPassword.KLPRSS_Val.value = '';
    }
    else if(changedData.NewPassword!=='') {
        const dataPswrd = getPswdrData(changedData.NewPassword);
        const response = await kscApi.DataProtectionApi.ProtectDataGlobally({ pData:dataPswrd });
        if(response.result!=='')
            DefaultProxy.ProxyAuth.KLPRSS_Val.AuthPassword.KLPRSS_Val.value = response.result;
    }

    policiesCache.ConnectionPolicy.Policy.KLPRSS_Val.ConnectionOptions = changedData.ConnectionOptions;
    policiesCache.BLPolicy.Policy.KLPRSS_Val.UseNagentAsActivationProxy = changedData.UseNagentAsActivationProxy;
}

module.exports = {
    sections,
    toView,
    toRawEx
};
