const lo = require('lodash');
const hlp     = require('../../../../helpers')
const utils   = require('../../../../utils')
const utilsTs = require('../../../utils/ts.js')
const hlpSchedule = require('../Schedule/schedule.js')

const sections = ['WebTrafficMonitorPolicy','PredefinedTasksPolicy'];
const formModeValues = ['ProxyMode','ServerPort','Uri','EnableHttps','TlsVersion','BlockUntrustedCert','ProxyPort','UseExtProxyServer','ExtProxyServerAddress','ExtProxyServerPort']
const formProtectValues = ['UseHeuristicAnalyzer','HeuristicAnalyzerLevel','ScanArchived','ScanPlainMail','InfectedAction','SuspiciousAction',
    'UseSizeLimit','SizeLimit','UseAdvancedActions'];
const formWebValues = ['UseOutlookPlugin','EnableWmuf','EnableAntiPhishing','UseKsn','UseSandbox'];
const formWcValues = ['UseUrlRules', 'UseCertificates', 'UrlRules', 'CertificateSearchPatterns'];
const formCategorizerValues = ['UseContentCategories', 'ContentCategories'];

function toView (policiesCache) {
    let data = {
        Form_TS_Mode: hlp.customToView(policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, formModeValues),
        Form_TS_Protect: hlp.customToView(policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, formProtectValues),
        Form_TS_Web: hlp.customToView(policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, formWebValues),
        Form_TS_Wc: hlp.customToView(policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, formWcValues),
        Form_TS_Categorization:hlp.customToView(policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, formCategorizerValues),
        Form_TS_Schedule: hlpSchedule.toView(policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.TrafficMonitorStartOptions)
    }
    data.Form_TS_Protect.ScanSettings = policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val.Base;
    data.Form_TS_Protect.uiFields = {
        ExtList: data.Form_TS_Protect.ScanSettings.KLPRSS_Val.ScanExtensionMasks.KLPRSS_Val[0].join(),
        ExcludeThreats:data.Form_TS_Protect.ScanSettings.KLPRSS_Val.ExcludeThreats.KLPRSS_Val[0].join(),
        Bigger:(data.Form_TS_Protect.SizeLimit.KLPRSS_Val[0]/(1024*1024))
    };
    data.Form_TS_Wc.uiFields = utilsTs.toViewWcRules(data.Form_TS_Wc);
    data.Form_TS_Web.UseTz = policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val.Base.KLPRSS_Val.UseTrustedZoneSettings;
    data.Form_TS_Categorization.uiFields = utilsTs.toViewCategories(data.Form_TS_Categorization);
    return data
}

function toRaw (changedData, policiesCache) {
    // mode
    Object.assign( policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_TS_Mode, formModeValues));
    // protection
    changedData.Form_TS_Protect.SizeLimit.KLPRSS_Val[0] = changedData.Form_TS_Protect.uiFields.Bigger*(1024*1024)
    Object.assign( policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_TS_Protect, formProtectValues));
    changedData.Form_TS_Protect.ScanSettings.KLPRSS_Val.ScanExtensionMasks.KLPRSS_Val[0] = utils.normalizedArrayMasks(changedData.Form_TS_Protect.uiFields.ExtList);
    changedData.Form_TS_Protect.ScanSettings.KLPRSS_Val.ExcludeThreats.KLPRSS_Val[0] = utils.normalizedArrayMasks(changedData.Form_TS_Protect.uiFields.ExcludeThreats);
    const bMandatoryProtect = changedData.Form_TS_Protect.ScanArchived.KLPRSS_Mnd;
    changedData.Form_TS_Protect.ScanSettings.KLPRSS_Val.ScanExtensionMasks.KLPRSS_Mnd = bMandatoryProtect;
    changedData.Form_TS_Protect.ScanSettings.KLPRSS_Val.ExcludeThreats.KLPRSS_Mnd = bMandatoryProtect;
    changedData.Form_TS_Protect.ScanSettings.KLPRSS_Val.UseExcludeThreats.KLPRSS_Mnd = bMandatoryProtect;
    changedData.Form_TS_Protect.ScanSettings.KLPRSS_Val.TimeLimit.KLPRSS_Mnd = bMandatoryProtect;
    changedData.Form_TS_Protect.ScanSettings.KLPRSS_Val.UseTimeLimit.KLPRSS_Mnd = bMandatoryProtect;

    Object.assign(policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val.Base, changedData.Form_TS_Protect.ScanSettings);
    //Web
    Object.assign( policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_TS_Web, formWebValues));
    policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val.Base.KLPRSS_Val.UseTrustedZoneSettings = changedData.Form_TS_Web.UseTz;
    // Wc
    const WcRules = utilsTs.toRowWcRules(changedData.Form_TS_Wc);
    Object.assign( policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, lo.pick(WcRules, formWcValues));
    // Categorization
    changedData.Form_TS_Categorization = utilsTs.toRowCategories(changedData.Form_TS_Categorization);
    Object.assign( policiesCache.WebTrafficMonitorPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_TS_Categorization, formCategorizerValues));
    // schedule
    hlpSchedule.toRaw(changedData.Form_TS_Schedule, policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.TrafficMonitorStartOptions);
}

module.exports = {
    sections,
    toView,
    toRaw
};
