const buffer = require('buffer');
const hlp = require('../../../../helpers');

const sections = ['ConnectionSettings','BLSettings'];
const formValues = ['BypassLocalAddresses','DefaultProxy','ProxyUsage'];
const proxyAuthValues = ['ProxyAuth','ProxyAuthType','ProxyPort','ProxyServer'];

function getPswdrData(pswrd) {
    let bytes = Buffer.from(pswrd + String.fromCharCode(0), 'utf-8');
    let bufStr16 = buffer.transcode(bytes, 'utf8', 'utf16le');
    return bufStr16.toString('base64');
}

function toView (settCache, activePolicyCache) {
    const ConnOptionsLocks = hlp.GetMergedLocalSettingsKey(settCache.ConnectionSettings,activePolicyCache.ConnectionSettings,'ConnectionOptions');
    const ConnectionOptions = ConnOptionsLocks.KLPRSS_Val[0];
    const useNagentAsActivationProxy = hlp.GetMergedLocalSettingsKey(settCache.BLSettings, activePolicyCache.BLSettings, 'UseNagentAsActivationProxy');
    // obsoleted case
    if(ConnectionOptions.DefaultProxy.KLPRSS_Val=== undefined){
        const DefaultProxy = ConnectionOptions.DefaultProxy;
        const ProxyAuth = DefaultProxy.ProxyAuth;
        return {
            ConnectionOptions:ConnOptionsLocks,
            ProxyMode: ConnectionOptions.ProxyUsage,
            ProxyServer: DefaultProxy.ProxyServer,
            ProxyPort: DefaultProxy.ProxyPort,
            BypassLocalAddresses: ConnectionOptions.BypassLocalAddresses,
            ProxyAuthMode: ProxyAuth.AuthType,
            ProxyAuthUser: ProxyAuth.AuthUser,
            ProxyPassword: ProxyAuth.AuthPassword,
            NewPassword: '',
            UseEmptyPassword: ProxyAuth.AuthPassword.value!=='' ? false : true,
            PasswordWasSet: ProxyAuth.AuthPassword.value!=='' ? true : false,
            UseNagentAsActivationProxy: useNagentAsActivationProxy,
            LocalSettings: true
        }
    }

    const DefaultProxy = ConnectionOptions.DefaultProxy.KLPRSS_Val;
    const ProxyAuth = DefaultProxy.ProxyAuth.KLPRSS_Val;
    const ProxyAuthP = ProxyAuth.AuthPassword.KLPRSS_Val;
    const data = {
        ConnectionOptions:ConnOptionsLocks,
        ProxyMode: ConnectionOptions.ProxyUsage.KLPRSS_Val,
        ProxyServer: DefaultProxy.ProxyServer.KLPRSS_Val,
        ProxyPort: DefaultProxy.ProxyPort.KLPRSS_Val,
        BypassLocalAddresses: ConnectionOptions.BypassLocalAddresses.KLPRSS_Val,
        ProxyAuthMode: ProxyAuth.AuthType.KLPRSS_Val,
        ProxyAuthUser: ProxyAuth.AuthUser.KLPRSS_Val,
        ProxyPassword: ProxyAuthP,
        NewPassword: '',
        UseEmptyPassword: ProxyAuthP.value!=='' ? false : true,
        PasswordWasSet: ProxyAuthP.value!=='' ? true : false,
        UseNagentAsActivationProxy: useNagentAsActivationProxy,
        LocalSettings: true
    }
    return data;
}

async function toRawEx (kscApi, changedData, settCache) {
    hlp.ItemToLocalSettings(changedData);
    let ConnectionOptions = changedData.ConnectionOptions;
    let DefaultProxy = ConnectionOptions.DefaultProxy;
    ConnectionOptions.ProxyUsage = changedData.ProxyMode;
    ConnectionOptions.BypassLocalAddresses = changedData.BypassLocalAddresses;
    DefaultProxy.ProxyServer = changedData.ProxyServer;
    DefaultProxy.ProxyPort = changedData.ProxyPort;
    DefaultProxy.ProxyAuth.AuthType = changedData.ProxyAuthMode;
    DefaultProxy.ProxyAuth.AuthUser = changedData.ProxyAuthUser;
    if(changedData.UseEmptyPassword){
        DefaultProxy.ProxyAuth.AuthPassword.value = ''
    }
    else if(changedData.NewPassword!=='') {
        const dataPswrd = getPswdrData(changedData.NewPassword);
        const response = await kscApi.DataProtectionApi.ProtectDataGlobally({ pData:dataPswrd });
        if(response.result!=='')
            DefaultProxy.ProxyAuth.AuthPassword.value = response.result;
    }
    Object.assign(settCache.ConnectionSettings.ConnectionOptions, changedData.ConnectionOptions);
    Object.assign(settCache.BLSettings.UseNagentAsActivationProxy, changedData.UseNagentAsActivationProxy);
}

module.exports = {
    sections,
    toView,
    toRawEx
};
