const hlpDef = require('./default/policy.js');

const OriginTypeEnum = { '':0, 'Default':1, 'User': 2, 'Auto':3 };
const TargetTypeEnum = {'Binary':1, 'ScriptOrMsi': 2};
const ActionTypeEnum = { 'Allow':1, 'Deny':2, 'Audit':3};
const ConditionTypeEnum = { 'Certificate': 1, 'Sha256': 2, 'Path': 3 };
const EMPTY_THUMBPRINT = '0000000000000000000000000000000000000000';
const EMPTY_SHA256 = '0000000000000000000000000000000000000000000000000000000000000000';
const BinaryEngineType		= 0x80000000;
const ScriptOrMsiEngineType	= 0x40000000;


const RuleConverter = {
    'CertificateRule': {
        getRule: function (item) {
            const source = (item.CertificateRuleCondition!== undefined && item.CertificateRuleCondition[0].$!==undefined) ?
                item.CertificateRuleCondition[0].$ : {UseSubject:false, Subject: "", UseThumbprint:false, Thumbprint: ""}
            let rule = {
                Name: getTested(item.$, "Name", ""),
                Origin: OriginTypeEnum[getTested(item.$, "Origin", "Default")],
                Target: TargetTypeEnum[getTested(item.$, "Target", "ScriptOrMsi")],
                Action: ActionTypeEnum[getTested(item.$, "Action", "Allow")],
                UserOrGroupSid: getTested(item.$, "UserOrGroupSid", ""),
                RuleCondition: ConditionTypeEnum.Certificate,
                CertificateRuleCondition: {
                    UseSubject:source.UseSubject==='true'? true : false,
                    Subject: source.Subject,
                    UseThumbprint:source.UseThumbprint==='true'? true : false,
                    Thumbprint: source.Thumbprint
                },
                HashRuleCondition: {Hash: ""},
                PathRuleCondition: {Path: ""},
                Exclusions: toUiRileExclusions(item.Exclusions)
            }
            return rule
        }
    },
    'CertificateRuleCondition': {
        getExclusion : function(item) {
            const source = item.$
            return {
                RuleCondition: ConditionTypeEnum.Certificate,
                CertificateRuleCondition: {
                    UseSubject:source.UseSubject==='true'? true : false,
                    Subject: source.Subject,
                    UseThumbprint:source.UseThumbprint==='true'? true : false,
                    Thumbprint: source.Thumbprint
                },
                HashRuleCondition: {Hash: ""},
                PathRuleCondition: {Path: ""}
            }
        }
    },
    'HashRule': {
        getRule: function (item) {
            const source = (item.HashRuleCondition!== undefined && item.HashRuleCondition[0].$!==undefined) ?
                item.HashRuleCondition[0].$ : {Hash: ""}
            let rule = {
                Name: getTested(item.$, "Name", ""),
                Origin: OriginTypeEnum[getTested(item.$, "Origin", "Default")],
                Target: TargetTypeEnum[getTested(item.$, "Target", "ScriptOrMsi")],
                Action: ActionTypeEnum[getTested(item.$, "Action", "Allow")],
                UserOrGroupSid: getTested(item.$, "UserOrGroupSid", ""),
                RuleCondition: ConditionTypeEnum.Sha256,
                CertificateRuleCondition: {UseSubject:false, Subject: "", UseThumbprint:false, Thumbprint: ""},
                HashRuleCondition: { Hash: source.Hash },
                PathRuleCondition: {Path: ""},
                Exclusions: toUiRileExclusions(item.Exclusions)
            }
            return rule
        }
    },
    'HashRuleCondition': {
        getExclusion : function(item) {
            return {
                RuleCondition: ConditionTypeEnum.Sha256,
                CertificateRuleCondition: { UseSubject: false, Subject: "", UseThumbprint: false, Thumbprint:"" },
                HashRuleCondition: {Hash: getTested(item.$, "Hash", "")},
                PathRuleCondition: {Path: ""}
            }
        }
    },
    'PathRule': {
        getRule: function (item) {
            const source = (item.PathRuleCondition!== undefined && item.PathRuleCondition[0].$!==undefined) ?
                item.PathRuleCondition[0].$ : {Path: ""}
            let rule = {
                Name: getTested(item.$, "Name", ""),
                Origin: OriginTypeEnum[getTested(item.$, "Origin", "Default")],
                Target: TargetTypeEnum[getTested(item.$, "Target", "ScriptOrMsi")],
                Action: ActionTypeEnum[getTested(item.$, "Action", "Allow")],
                UserOrGroupSid: getTested(item.$, "UserOrGroupSid", ""),
                RuleCondition: ConditionTypeEnum.Path,
                CertificateRuleCondition: {UseSubject:false, Subject: "", UseThumbprint:false, Thumbprint: ""},
                HashRuleCondition: {Hash: ""},
                PathRuleCondition: {Path: getTested(source, "Path", "")},
                Exclusions: toUiRileExclusions(item.Exclusions)
            }
            return rule
        }
    },
    'PathRuleCondition': {
        getExclusion : function(item) {
            return {
                RuleCondition: ConditionTypeEnum.Path,
                CertificateRuleCondition: { UseSubject:false, Subject: "", UseThumbprint:false, Thumbprint:"" },
                HashRuleCondition: {Hash: ""},
                PathRuleCondition: {Path: getTested(item.$, "Path", "")}
            }
        }
    }
}

function getTested(source, key, defValue) {
    return source[key]!==undefined ? source[key] : defValue
}

function AddExclusionByType(ExclCollection, exclType, uiExcl) {
    if(ExclCollection[exclType]===undefined )
        return;
    for(let item of ExclCollection[exclType]) {
        uiExcl.push(RuleConverter[exclType].getExclusion(item))
    }
}

function toUiRileExclusions(jsonExclusions) {
    let exclusions = []
    if(jsonExclusions===undefined || jsonExclusions[0]===undefined)
        return exclusions
    AddExclusionByType(jsonExclusions[0], 'CertificateRuleCondition', exclusions)
    AddExclusionByType(jsonExclusions[0], 'HashRuleCondition', exclusions)
    AddExclusionByType(jsonExclusions[0], 'PathRuleCondition', exclusions)
    return exclusions
}

function AddPepRilesByType(RuleCollection, ruleType, uiRules) {
    if(RuleCollection[ruleType]===undefined )
        return;
    for(let item of RuleCollection[ruleType]) {
        uiRules.push(RuleConverter[ruleType].getRule(item))
    }
}

function toPepUiRiles(jsonRules) {
    let uiRules = []
    if(!jsonRules || jsonRules.ApplicationControl===undefined || jsonRules.ApplicationControl.RuleCollection===undefined )
        return uiRules;
    const Collection = jsonRules.ApplicationControl.RuleCollection[0];
    AddPepRilesByType(Collection, 'CertificateRule', uiRules)
    AddPepRilesByType(Collection, 'HashRule', uiRules)
    AddPepRilesByType(Collection, 'PathRule', uiRules)
    return uiRules
}

function exportJsonPepSettingsRule(json){
    let expItem = JSON.parse(json);
    if(expItem===undefined || expItem.ActionMask===undefined)
        return {};
    let rule = hlpDef.defaultApplicationControlSettingsRule();
    rule.Origin = OriginTypeEnum['Auto'];
    rule.Action = ActionTypeEnum['Allow'];
    rule.Target = (expItem.ActionMask & BinaryEngineType) ?  1 : 2;
    rule.UserOrGroupSid = getTested(expItem, "User", 'S-1-1-0');

    if(expItem.Valid==='Yes'){
        rule.Name = expItem.ProductName + 'signed by' + expItem.Issuer;
        rule.RuleCondition = ConditionTypeEnum.Certificate;
        rule.CertificateRuleCondition = {
            UseSubject: true,
            Subject:expItem.Issuer,
            UseThumbprint:true,
            Thumbprint:getTested(expItem, "Thumbprint", EMPTY_THUMBPRINT)
        };
    }
    else{ //Create by SHA256
        rule.Name = getTested(expItem, "ObjectName", '');
        rule.RuleCondition = ConditionTypeEnum.Sha256;
        rule.HashRuleCondition = {Type: 2, Hash:getTested(expItem, "Sha256", EMPTY_SHA256)};
        rule.PathRuleCondition = {Path: getTested(expItem, "ObjectName", "")};
    }
    return rule;
}

function isPepRulesEqual(baseRule, newRule) {
    if(baseRule.RuleCondition !== newRule.RuleCondition)
        return false;

    if(baseRule.RuleCondition === ConditionTypeEnum.Certificate){
        let basehash = '';
        let newhash = '';
        if(baseRule.CertificateRuleCondition.UseSubject)
            basehash += baseRule.CertificateRuleCondition.Subject;
        if(baseRule.CertificateRuleCondition.UseThumbprint)
            basehash += baseRule.CertificateRuleCondition.Thumbprint;
        if(newRule.CertificateRuleCondition.UseSubject)
            newhash += newRule.CertificateRuleCondition.Subject;
        if(newRule.CertificateRuleCondition.UseThumbprint)
            newhash += newRule.CertificateRuleCondition.Thumbprint;
        return (basehash===newhash);
    }
    else if(baseRule.RuleCondition === ConditionTypeEnum.Sha256){
        return (baseRule.HashRuleCondition.Hash === newRule.HashRuleCondition.Hash);
    }
    else if(baseRule.RuleCondition === ConditionTypeEnum.Path){
        return (baseRule.PathRuleCondition.Path === newRule.PathRuleCondition.Path);
    }
    return false;
}

module.exports = {
    toPepUiRiles,
    exportJsonPepSettingsRule,
    isPepRulesEqual
}